
float Frequency = 243;
float4x4 Transforms[5];
float4 DiffuseColor = float4(6, 2, 1, 6);

texture Palette;
sampler2D PaletteSampler = sampler_state {
	texture = <Palette>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

texture AtmospherePalette;
sampler2D AtmospherePaletteSampler = sampler_state {
	texture = <AtmospherePalette>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

struct VertexShaderInput {
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
};

struct VertexShaderOutput {
    float4 Position  : POSITION0;
    float2 Location0 : TEXCOORD0;
    float2 Location1 : TEXCOORD1;
    float2 Location2 : TEXCOORD2;
    float2 Location3 : TEXCOORD3;
    float2 Location4 : TEXCOORD4;
    float2 TexCoord : TEXCOORD5;
};

VertexShaderOutput VSNoise(VertexShaderInput input, uniform const int octave) {
    VertexShaderOutput output = (VertexShaderOutput)0;

    output.Position = input.Position;
    output.TexCoord = input.TexCoord;
	if (1 <= octave) output.Location0 = mul(float4(input.TexCoord, 0, 1), Transforms[0]);
	if (2 <= octave) output.Location1 = mul(float4(input.TexCoord, 0, 1), Transforms[1]);
	if (3 <= octave) output.Location2 = mul(float4(input.TexCoord, 0, 1), Transforms[2]);
	if (4 <= octave) output.Location3 = mul(float4(input.TexCoord, 0, 1), Transforms[3]);
	if (5 <= octave) output.Location4 = mul(float4(input.TexCoord, 0, 1), Transforms[4]);
    return output;
}


float4 hash4(float4 v, const float frequency) {
    return frac(v * v / frequency) * frequency;
}

float4 hash4xy(float4 x, float4 y, const float frequency) {
    return hash4(x + hash4(7 * y, frequency), frequency);
}

float4 gradient4(const float4 hash, float4 x, float4 y) {
   // JԂl -0.375, -0.125, 0.125, 0.375 //
   float4 h = frac(hash / 4.0) - 0.375;

   float4 sy = 2 * step(h, 0) - 1;
   float4 sx = 2 * step(abs(h), 0.25) - 1;

   // xy͓ւĂ悢 //
   return y * sy + x * sx;
}

float noise2D(const float2 position, const float frequency) {
   // mCYʒuV[h̒[ x, y, x+1, y+1 //
   float4 f = float4(frac(position), frac(position) - 1);

   // mCYʒuV[h̐ x, y, x+1, y+1 //
   float4 i = float4(floor(position), floor(position) + 1);

   // X[Y⊮ //
   float2 a = f.xy * f.xy * (3 - 2 * f.xy);

   // R[i[(x, y), (x+1, y), (x, y+1), (x+1, y+1)̗lvZ //
   float4 hash = hash4xy(i.xzxz, i.yyww, frequency);

   // eR[i[̌z //
   float4 g = gradient4(hash, f.xzxz, f.yyww);

   // R[i[̕⊮ //
   return lerp(lerp(g.x, g.y, a.x), lerp(g.z, g.w, a.x), a.y);
}


float PSNoise1(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float alpha = 0;
    if (1 <= octave) alpha += (1.0f /  2) * noise2D(input.Location0, Frequency);
    if (2 <= octave) alpha += (1.0f /  4) * noise2D(input.Location1, Frequency);
    if (3 <= octave) alpha += (1.0f /  8) * noise2D(input.Location2, Frequency);
    if (4 <= octave) alpha += (1.0f / 16) * noise2D(input.Location3, Frequency);
    if (5 <= octave) alpha += (1.0f / 32) * noise2D(input.Location4, Frequency);
    return alpha;
}

float4 PSNoise(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float alpha = PSNoise1(input, octave);
	return (alpha *0.5 + 0.5).xxxx;
}

float4 ocean = float4(0.2, 0.5, 1.0, 1.0);
float4 terrain = float4(0.5, 0.2, 0.1, 1.0);

float4 PSAtlas(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float alpha = PSNoise1(input, octave);
	return (alpha < 0) ? ocean: terrain;
}

float4 PSFire(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float alpha = PSNoise1(input, octave);
	alpha = alpha * 1.0 + 0.45;
    float4 color = DiffuseColor;
    color.rgba *= alpha;
    return color;
}

float4 Blend(float4 a, float4 b) {
	return lerp(a, b, b.a);
}
float4 Add(float4 a, float4 b) {
	return lerp(a, a + b, b.a);
}
float4 Subtract(float4 a, float4 b) {
	return lerp(a, a - b, b.a);
}
float4 Multiply(float4 a, float4 b) {
	return lerp(a, a * b, b.a);
}
float4 Screen(float4 a, float4 b) {
	return lerp(a, 1 - (1 - a) * (1 - b), b.a);
}
float4 Overlay(float4 a, float4 b) {
	return lerp(a, (a < 0.5) ? 2 * a * b : 1 - 2 * (1 - a) * (1 - b), b.a);
}
float4 Hardlight(float4 a, float4 b) {
	return lerp(a, (b < 0.5) ? 2 * a * b : 1 - 2 * (1 - a) * (1 - b), b.a);
}
float4 Dodge(float4 a, float4 b) {
	return lerp(a, (b >= 1) ? 1 : a / (1 - b), b.a);
}
float4 Burn(float4 a, float4 b) {
	return lerp(a, (b <= 0) ? 0 : 1 - (1 - a) / b, b.a);
}

float4 PSCloud(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float displacement = PSNoise1(input, octave);
	VertexShaderOutput i = input;
	i.Location0 += displacement.xx * -0.1;
	i.Location1 += displacement.xx * -0.2;
	i.Location2 += displacement.xx * -0.5;
	i.Location3 += displacement.xx * -1;
	i.Location4 += displacement.xx * -8;
	float alpha = PSNoise1(i, octave);
	float4 color = tex2Dlod(PaletteSampler, float4(alpha * 0.5 + 0.5, 0, 0, 0));
	float4 atmosphere = tex2Dlod(AtmospherePaletteSampler, float4(input.TexCoord, 0, 0));
    return Hardlight(color, atmosphere);
}

float4 PSStripe(VertexShaderOutput input, uniform const int octave) : COLOR0 {
	float alpha = PSNoise1(input, octave);
	alpha = sin(input.Location0.x + alpha) * 1.0 + 0.45;
    float4 color = DiffuseColor;
    color.rgba *= alpha;
    return color;
}

technique Noise1 {
    pass Noise {
        VertexShader = compile vs_1_1 VSNoise(1);
        PixelShader = compile ps_2_0 PSNoise(1);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Noise2 {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(2);
        PixelShader = compile ps_3_0 PSNoise(2);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Noise3 {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(3);
        PixelShader = compile ps_3_0 PSNoise(3);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Noise4 {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(4);
        PixelShader = compile ps_3_0 PSNoise(4);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Noise5 {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(5);
        PixelShader = compile ps_3_0 PSNoise(5);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Fire {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(5);
        PixelShader = compile ps_3_0 PSFire(5);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Stripe {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(5);
        PixelShader = compile ps_3_0 PSStripe(5);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Atlas {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(5);
        PixelShader = compile ps_3_0 PSAtlas(5);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
technique Cloud {
    pass Noise {
        VertexShader = compile vs_2_0 VSNoise(5);
        PixelShader = compile ps_3_0 PSCloud(5);
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
    }
}
