using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace XELF.Effects.Noise {

	public class NoiseGame : Microsoft.Xna.Framework.Game {
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;

		static Texture2D CreateGradation(GraphicsDevice graphicsDevice) {
			var colors = new Vector4[]{
				new Vector4(0x77 / 255.0f, 0x77 / 255.0f, 0xa4 / 255.0f, 0xff / 255.0f),
				new Vector4(0xf3 / 255.0f, 0xf8 / 255.0f, 0xff / 255.0f, 0xff / 255.0f),
				new Vector4(0xcc / 255.0f, 0xdd / 255.0f, 0xff / 255.0f, 0xff / 255.0f),
				new Vector4(0x21 / 255.0f, 0x5f / 255.0f, 0xdf / 255.0f, 0xff / 255.0f),
			};
			var ratios = new float[] { 0, 80 / 255.0f, 120 / 255.0f, 145 / 255.0f };
			var data = Gradient.CreateLinearGradient(colors, ratios, 256);
			var texture = new Texture2D(graphicsDevice, data.Length, 1, 1, TextureUsage.None, SurfaceFormat.Vector4);
			texture.SetData(data);
			return texture;
		}
		static Texture2D CreateAtmosphere(GraphicsDevice graphicsDevice) {
			var colors = new Vector4[]{
				new Vector4(0.300f, 0.300f, 0.300f, 1.000f),
				new Vector4(0.800f, 0.800f, 1.000f, 1.000f),
			};
			var ratios = new float[] { 0, 1 };
			var data = Gradient.CreateLinearGradient(colors, ratios, 256);
			var texture = new Texture2D(graphicsDevice, 1, data.Length, 1, TextureUsage.None, SurfaceFormat.Vector4);
			texture.SetData(data);
			return texture;
		}

		Texture2D palette;
		Texture2D atmospherePalette;

		public NoiseGame() {
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
		}

		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			quad = new QuadFiller(GraphicsDevice);
			effect = Content.Load<Effect>("Noise");

			palette = CreateGradation(GraphicsDevice);
			atmospherePalette = CreateAtmosphere(GraphicsDevice);
			effect.Parameters["Palette"].SetValue(palette);
			effect.Parameters["AtmospherePalette"].SetValue(atmospherePalette);
			speed = cloudSpeed;
		}

		const int MaximumOctaves = 5;
		Vector3[] offset = new Vector3[MaximumOctaves];
		Matrix[] transforms = new Matrix[MaximumOctaves];
		float Frequency = 257 * 257;//243;
		Effect effect;
		QuadFiller quad;

		KeyboardState previousKeyState;
		KeyboardState currentKeyState;
		bool IsKeyPress(Keys key) {
			return currentKeyState.IsKeyDown(key) && previousKeyState.IsKeyDown(key);
		}

		enum Style {
			Noise1,
			Noise2,
			Noise3,
			Noise4,
			Noise5,
			Fire,
			Stripe,
			Atlas,
			Cloud,
		}

		Style style = Style.Cloud;
		Vector2 scale = new Vector2(0.5f, 1.1f);

		void HandleInput() {
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			previousKeyState = currentKeyState;
			currentKeyState = Keyboard.GetState();
			if (IsKeyPress(Keys.C)) {
				style = Style.Cloud;
				scale.X = 0.5f; scale.Y = 1.1f;
				speed = cloudSpeed;
			}
			if (IsKeyPress(Keys.F)) {
				style = Style.Fire;
				scale.X = 2; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.S)) {
				style = Style.Stripe;
				scale.X = 2; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.A)) {
				style = Style.Atlas;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.F1)) {
				style = Style.Noise1;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.F2)) {
				style = Style.Noise2;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.F3)) {
				style = Style.Noise3;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.F4)) {
				style = Style.Noise4;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}
			if (IsKeyPress(Keys.F5)) {
				style = Style.Noise5;
				scale.X = 1; scale.Y = 1;
				speed = fireSpeed;
			}

		}

		Vector3[] speed;
		static readonly Vector3[] fireSpeed ={
							 new Vector3(+1.0f * 0.008f, +1.0f * 0.030f, 0),
							 new Vector3(+3.0f * 0.008f, +1.5f * 0.030f, 0),
							 new Vector3(+2.0f * 0.008f, +0.5f * 0.030f, 0),
							 new Vector3(+1.0f * 0.008f, +2.0f * 0.030f, 0),
							 new Vector3(-1.5f * 0.008f, +1.6f * 0.030f, 0),
						};
		static readonly Vector3[] cloudSpeed ={
							 new Vector3(+0.00067f, +0.00000f, 0),
							 new Vector3(+0.00061f, +0.00001f, 0),
							 new Vector3(-0.00025f, -0.00040f, 0),
							 new Vector3(+0.00020f, +0.00081f, 0),
							 new Vector3(-0.00041f, +0.0090f, 0),
						};

		protected override void Update(GameTime gameTime) {
			HandleInput();

			for (int i = 0; i < offset.Length; i++) {
				Vector3.Add(ref offset[i], ref speed[i], out offset[i]);
			}
			float a = GraphicsDevice.Viewport.AspectRatio * scale.X;
			float b = scale.Y;
			transforms[0] = Matrix.CreateScale(4 * a, 4 * b, 1) * Matrix.CreateTranslation(offset[0]);
			transforms[1] = Matrix.CreateScale(8 * a, 8 * b, 1) * Matrix.CreateTranslation(offset[1]);
			transforms[2] = Matrix.CreateScale(16 * a, 16 * b, 1) * Matrix.CreateTranslation(offset[2]);
			transforms[3] = Matrix.CreateScale(32 * a, 32 * b, 1) * Matrix.CreateTranslation(offset[3]);
			transforms[4] = Matrix.CreateScale(64 * a, 64 * b, 1) * Matrix.CreateTranslation(offset[4]);
			effect.CurrentTechnique = effect.Techniques[style.ToString()];
			effect.Parameters["Frequency"].SetValue(Frequency);
			effect.Parameters["Transforms"].SetValue(transforms);

			base.Update(gameTime);
		}
		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.Black);
			quad.Fill(effect);
			base.Draw(gameTime);
		}
	}

	class QuadFiller : IDisposable {
		readonly GraphicsDevice graphicsDevice;
		readonly VertexDeclaration declaration;
		public QuadFiller(GraphicsDevice graphicsDevice) {
			this.graphicsDevice = graphicsDevice;
			declaration = new VertexDeclaration(graphicsDevice, VertexPositionTexture.VertexElements);
		}
		readonly VertexPositionTexture[] vertices = new VertexPositionTexture[] {
			new VertexPositionTexture(new Vector3(-1, +1, 0), new Vector2(0, 0)),
			new VertexPositionTexture(new Vector3(+1, +1, 0), new Vector2(1, 0)),
			new VertexPositionTexture(new Vector3(-1, -1, 0), new Vector2(0, 1)),
			new VertexPositionTexture(new Vector3(+1, -1, 0), new Vector2(1, 1)),
		};
		public void Dispose() {
			declaration.Dispose();
		}
		public void Fill(Effect effect) {
			effect.Begin();
			foreach (var pass in effect.CurrentTechnique.Passes) {
				pass.Begin();
				Fill();
				pass.End();
			}
			effect.End();
		}
		public void Fill() {
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
		public void Fill(Effect effect, float x0, float y0, float x1, float y1) {
			effect.Begin(SaveStateMode.SaveState);
			foreach (var pass in effect.CurrentTechnique.Passes) {
				pass.Begin();
				Fill(x0, y0, x1, y1);
				pass.End();
			}
			effect.End();
		}
		public void Fill(float x0, float y0, float x1, float y1) {
			VertexPositionTexture[] vertices = new VertexPositionTexture[] {
				new VertexPositionTexture(new Vector3(x0, y1, 0), new Vector2(0, 0)),
				new VertexPositionTexture(new Vector3(x1, y1, 0), new Vector2(1, 0)),
				new VertexPositionTexture(new Vector3(x0, y0, 0), new Vector2(0, 1)),
				new VertexPositionTexture(new Vector3(x1, y0, 0), new Vector2(1, 1)),
			};
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
	}

}
